;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3cdc9620f65bfba46cd8d122d8f7db26 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x9a8288e413e7dac3ef2b3dbbc41e704c . "./code.cg2js/cg-printing.fasl")
(#x7f9d1b520a1fcc48046f1e3103c982a4 . "./code.cg2js/cg-caret.fasl")
(#x7ee2e982b0831c0edd461612d8cffdfb . "./code.cg2js/cg-item-list.fasl")
(#xe1f18837f6235be0963e44405337c06c . "./code.cg2js/cg-progress-indicator.fasl")
(#xcb2b472302d873512d9783c3c17dc3d9 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x48ccdb1c3c0e5dcd4d753c4a36a9214e . "./code.cg2js/cg-outline.fasl")
(#x818dbf7a4cccb55d528d258702df891f . "./code.cg2js/cg-palette.fasl")
(#x398983733ed04d974fdde81086235cea . "./code.cg2js/cg-selected-object.fasl")
(#xf60c0455534230442d6c8d1445208a1b . "./code.cg2js/cg-status-bar.fasl")
(#x0c2b97d0938007be036786def30cc749 . "./code.cg2js/cg-wrap-string.fasl")
(#xe6a2f9217887d7036afce87c9fe37fbf . "./code.cg2js/cg-fill-texture.fasl")
(#x570c1d629e7a34a2e45003b8e420e5e8 . "./code.cg2js/cg-layout.fasl")
(#xf4f43cfee68d6b854f40b9ff0f22ae60 . "./code.cg2js/cg-timer.fasl")
(#x4b0663d071d8e89d45753f514a46660b . "./code.cg2js/cg-dialog-item.fasl")
(#xdced0150d983a47b3cc6792bf3cc19ef . "./code.cg2js/cg-button.fasl")
(#x464f854776516ca2de8e60f8f2d5e18c . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x207dfa443201674e170ae0112be47665 . "./code.cg2js/cg-tray.fasl")
(#x5be44fd2ce6ea577293e01ce94580f73 . "./code.cg2js/cg-plot-widget.fasl")
(#xf7b2576238c01415a0684a3bc55c642c . "./code.cg2js/cg-pixmap.fasl")
(#x3f59e4ba314b1e4268d17b40be66a081 . "./code.cg2js/cg-font-dialog.fasl")
(#x4b9d3bafa641923c38c3d8ed1a43f84b . "./code.cg2js/cg-os-widget.fasl")
(#xd7712bf47831cc65e8fc5e91da288719 . "./code.cg2js/cg-text-or-combo.fasl")
(#xccd8b0e0623fe17ca14e98fb0bf1f46a . "./code.cg2js/cg-trackbar.fasl")
(#x42a76ccd68f58b97a517be6737243801 . "./code.cg2js/cg-up-down-control.fasl")
(#xb329dea22d9982a199c8f007cab24aa6 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x1cdb6382d22bea8a05605f1c8202bfde . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf2a3072b07b7316f47f96ea937dbc7fe . "./code.cg2js/cg-intersect-posbox.fasl")
(#x5c042163978e440c58f8ac3a63409af7 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xcb3d0625fe59e90ee60105571f9fca7e . "./code.cg2js/cg-chart-widget.fasl")
(#x81220162f5964450842109841d5b0893 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x61b10f4834f8953d196bac3bd34cd100 . "./code.cg2js/cg-group-box.fasl")
(#x9c4ba1d071934dcf48c0dcfe70e03cdb . "./code.cg2js/cg-static-text.fasl")
(#x4c0034eaa933dd244baf37c5c6207969 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xee19886875759f5187321b733c2f0c80 . "./code.cg2js/cg-color-dialog.fasl")
(#x99e63230ab4141bf9b474728521de02d . "./code.cg2js/cg-intersect.fasl")
(#x32c67841434e5bef62cd7fbbebb69849 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x88d8ee4360caf57d3b6f527caf63f9b9 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xfd8c8b0ca05b4824cf16b5cf2921f9da . "./code.cg2js/cg-html-dialog.fasl")
(#xbbfa5fb9c4887a6b13ecd884cc4dcec4 . "./code.cg2js/cg-lettered-menu.fasl")
(#xf966e969025357acfa5f7dde32330203 . "./code.cg2js/cg-class-grid.fasl")
(#xe5317d34f8f50d7f7f9fb2acee79dbab . "./code.cg2js/cg-pixmap-printing.fasl")
(#x27300b89004a96cc7148aaf54e36a1b1 . "./code.cg2js/cg-calendar.fasl")
(#xcb47f4683c074dd5aab58dd1118a161b . "./code.cg2js/cg-editable-text.fasl")
(#x58077b0e5c8cbea397a61b4811c2e2e1 . "./code.cg2js/cg-property.fasl")
(#x94c575f7d2912433498f11e74fe996e9 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x35b10583461cdc776974afa610227bda . "./code.cg2js/cg-grid-widget.fasl")
(#xf5ee602597638ead604d4b04a82841d3 . "./code.cg2js/cg-paren-matching.fasl")
(#x2afe218ac02b307c9bf4c70b3baeb544 . "./code.cg2js/cg-dev.fasl")
(#xb166587dd2c53c5309a5d7d11b488449 . "./code.cg2js/cg-drawable.fasl")
(#xf30c8323d3666e4481b5a8c9a40eb8bc . "./code.cg2js/cg-curve.fasl")
(#x8e3cb13f088f38b2bd7df81ee3927f18 . "./code.cg2js/cg-lisp-widget.fasl")
(#xfbcc7a0bedb8eb2c35fc4e2fe9e025e7 . "./code.cg2js/cg-dde-utils.fasl")
(#xe89d1292e83c80020a6f138aa79e2094 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x6a41fcf5b51d55bbc43e00f9c644d609 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x398350e5b67c0b7013e326a49c965251 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa7507dd475ebe689869e897d39a3d4ed . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xe3bfe5cb12dd963b23af7f83308c3041 . "./code.cg2js/cg-lamp.fasl")
(#x36b5d49eb193921135333a50b1f82737 . "./code.cg2js/cg-common-control.fasl")
(#x2f46956ed75c0562e9d9e6c644a07aec . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xaa9e5097b0f1535f85f404e9b01efc9a . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xff7ad027a57ff451e0fcfc96e3db0cd5 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xdffa0ccb26965c723ecf943e34162c6c . "./code.cg2js/cg-project-window.fasl")
(#xd934a5d7b75d49797b1725b87ae98a0d . "./code.cg2js/cg-sample-file-menu.fasl")
(#xd8a8112c5d53a8583661e79ae2a6d6e4 . "./code.cg2js/cg-class-support.fasl")
(#xac798715a76964c9786f08a4b5c8af11 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xd2c7411901f4fbea932156775d15939a . "./code.cg2js/cg-yes-no-list.fasl")
(#xf07f0169c119589b3ed1ec40cd7ad6a0 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x0b6e2389d1fe4c4f3bf1208af3aec5a2 . "./code.cg2js/cg-string-dialog.fasl")
(#xac74c9de391fd04931a9e43455461def . "./code.cg2js/cg-menu-tooltip.fasl")
(#xbcd7d98345067ecfbb03f41794d9ab44 . "./code.cg2js/cg-dropping-outline.fasl")
(#x78e1c875919d16dbc497e1e86c06f7c2 . "./code.cg2js/cg.fasl")
(#x04d0b38745d6cebe123f1fde6e06bb10 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa6e6d1a2b5bee7c2abb6185034bdea53 . "./code.cg2js/cg-scaling-stream.fasl")
(#xf1ebe2193d2b2c3070e466242ec1fbb1 . "./code.cg2js/cg-yes-no-string.fasl")
(#xc03c46e1f036ddcf5a2c0ee7df169d10 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xc776d0a19a8eaab72509db01af912a49 . "./code.cg2js/cg-web-browser.fasl")
(#x4b1638e10e4e9375ce9688f8b3bc1f60 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf9c98293a6949d988723596f28fb3a92 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x925bbcb180b72a34204844e996b3c1e0 . "./code.cg2js/cg-message-dialog.fasl")
(#x82811089790ba1a116fb431447a4dd82 . "./code.cg2js/cg-comtab.fasl")
(#x49bde303b5ae6e2ee5a37c2250501d68 . "./code.cg2js/cg-base.fasl")
(#x22671692c445e5fc1096271dda3eb1ad . "./code.cg2js/cg-pixmap-widget.fasl")
(#xe86365ddc92cc3d5a374d5e0754d3898 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x41774d4df809d53c282648199d0888e3 . "./code.cg2js/cg-utility-dialog.fasl")
(#x535498f414e1d1534b9a91537c30d2f9 . "./code.cg2js/cg-object-editor.fasl")
(#x66ee602a0efc024c15b02d2fc1065fd1 . "./code.cg2js/cg-edit-in-place.fasl")
(#xbcae33911827f2bf2990d708cf1650e0 . "./code.cg2js/cg-radio-button.fasl")
(#xd045f0d35570214bed0d3f8ce615513d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x1fd4338276e2844503a6456f967f1072 . "./code.cg2js/cg-nodes.fasl")
(#x7ac06db32d167c514c96891ed15f2235 . "./code.cg2js/cg-get-position.fasl")
(#x7e2f2f5addd28f3d0e795834894398c3 . "./code.cg2js/cg-directory-dialog.fasl")
(#xce5f9178dbd53da6829513a06afa493e . "./code.cg2js/cg-lisp-text.fasl")
(#x834ff3f8dc21d3e7cb0f6747779f3b7b . "./code.cg2js/cg-combo-box.fasl")
(#xda066efa8289a94c3b92461b4f91a010 . "./code.cg2js/cg-text-widget.fasl")
(#x2891caedbdee2461d1a18fdde662957d . "./code.cg2js/cg-list-view.fasl")
(#x4d546f175632be7e51b35ab7a64f7800 . "./code.cg2js/cg-graphics-context.fasl")
(#x391e89bd2560b0510510c0ea96121e5c . "./code.cg2js/cg-get-pixmap.fasl")
(#xc3e49adbb7193b4fefdfe1033870d7a6 . "./code.cg2js/cg-menu.fasl")
(#xdb5aacc7972c80564f59f77f59b091aa . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x54d23f065e91c0fdd3af8bf3ff9929c7 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8846a4c6f6d508f2b433f2ce47197cde . "./code.cg2js/cg-tooltip.fasl")
(#x243be9f8a0717cd97db8e3eed25faa56 . "./code.cg2js/cg-html-widget.fasl")
(#x6c0eeab7cc60c49a0130b44975f62a3a . "./code.cg2js/cg-choice-list.fasl")
(#x8a7403c7517e1af4fddbdd0f19df2db1 . "./code.cg2js/cg-icon.fasl")
(#xd95f4debb9cab5eed71289a8f2831222 . "./code.cg2js/cg-acache.fasl")
(#x7f8a87e5aeca365c24b13f86236d9232 . "./code.cg2js/cg-template-string.fasl")
(#x345c35c5900b9d470ecb15112ad3fd6a . "./code.cg2js/cg-bitmap-pane.fasl")
(#x471ff5f300f799df90ac6c5782803368 . "./code.cg2js/cg-toggling-widget.fasl")
(#xfb4ea36ca9732e6df7ee653a8146047e . "./code.cg2js/cg-tab-control.fasl")
(#xb2d1c8775ec968fe94012a9686827ede . "./code.cg2js/cg-split-bar.fasl")
(#x33818de88bebe93ab0e60d91bf1782f2 . "./code.cg2js/cg-picture-widget.fasl")
(#x1baef98c4debed1f8eeacd155c63e2ef . "./code.cg2js/cg-player.fasl")
(#xe13d756da4f723d90dbb0e572e20e49b . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x013fedfdd76e4de7fc21d84cdcc2b706 . "./code.cg2js/cg-choose-printer.fasl")
(#xcd09bd4e94c968d08160d2dfb11de093 . "./code.cg2js/cg-os-window.fasl")
(#x4ca5164bd295242e3644dad2ffb8b17b . "./code.cg2js/cg-scroll-bar.fasl")
(#xe24852bd716c89ad0340c552b531c65f . "./code.cg2js/cg-clipboard.fasl")
(#x65a54943f686da8041ae9fa49e646080 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe5dcb0cf6661ecab7f6adb07000945dd . "./code.cg2js/cg-hotspot.fasl")
(#x3a4dc3c4c84f4348db383f8bf714f433 . "./code.cg2js/cg-check-box.fasl")
(#xe36f42b99a8d4b89266e88b4dc5f2293 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x27cb413da7eb3c5453c9c56579dfaa13 . "./code.cg2js/cg-file-dialog.fasl")
(#x09bf4fe7084007b95b31921c782ffaa0 . "./code.cg2js/cg-toolbar.fasl")
(#x90a3429eebb0a9350bdf74ca99cd8707 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x80fb3ae5274263f554248fd6a574ca14 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xbcd61a89c8b68fea42b04e49724bf976 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3a7c7002e4ba9696c19873e7ff738964 . "./code.idejs/ide-project.fasl")
(#xaba7fcb9b9e56e0a2500c51a89b6210b . "./code.idejs/ide-intro.fasl")
(#x87f978065ac52f5dea4a2f0b724ed77e . "./code.idejs/ide-find-in-files.fasl")
(#x7642d319d045f036f142bb852cc45870 . "./code.idejs/ide-grapher.fasl")
(#xdfe9eea049d843ba0dadcd7e32428b62 . "./code.idejs/ide-inspector.fasl")
(#xfb11404f636d254877601616e12c658e . "./code.idejs/ide-editor.fasl")
(#x06911b60d6e610c04acb7e4ac1c4918c . "./code.idejs/ide-window-list.fasl")
(#x76816a21f23af9e485ae1df735780d4c . "./code.idejs/ide-process-browser.fasl")
(#x7654bba48640494e7f89fada9b5e844a . "./code.idejs/ide-patch-dialog.fasl")
(#xb927c31b3f0d92ad156f57161e0600ac . "./code.idejs/ide-class-grapher.fasl")
(#x4886bbca681df17ea3c452ebd6f28199 . "./code.idejs/ide-base.fasl")
(#x6b456d8b4621c2b17a3548111956cf0e . "./code.idejs/ide.fasl")
(#x1a7529ccd661178f25cbfbbecb80eff2 . "./code.idejs/ide-trace.fasl")
(#x7423c4f3c2cda3f3ef17ad8635fc1b4c . "./code.idejs/ide-form.fasl")
(#x93e636e7bf33fe6419ceb9bc82085d8e . "./code.idejs/ide-find-definitions.fasl")
(#xa30bbf68dacddc1a6a53c1bd64f4b540 . "./code.idejs/ide-apropos.fasl")
(#x2efe5c045b91b0f558ba80c60b86a9b1 . "./code.idejs/ide-class-browser.fasl")
(#xbdc7977a6e461eec45712dc7a20850d0 . "./code.idejs/ide-navigator.fasl")
(#xacbf912f3b6c5b02b11bd2802471864f . "./code.idejs/ide-clipboard.fasl")
(#x9ee3eb4043c8533b966b59275b6a1183 . "./code.idejs/ide-acache.fasl")
(#x6ba572e206e6dc33cf2cbec9b0744745 . "./code.idejs/ide-keyshort.fasl")
(#x2f8547ba9d651bbda220ae3e5f59f369 . "./code.idejs/ide-package-list.fasl")
(#xd052768e44e7790f511277e8aa09de31 . "./code.idejs/ide-dev.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
